<?php // -*- coding: utf-8 -*-

/** \file Db.inc
 * (October 25, 2015)
 *
 * \brief
 * Class to connect and deal to the MySQL database.
 *
 * <img src="db_opiquotations_graph.png" width="470" height="809" border="0" alt="[DB opiquotations graph]" />
 *
 * Piece of OPiQuotations.
 * https://bitbucket.org/OPiMedia/opiquotations
 *
 * GPLv3 --- Copyright (C) 2014, 2015 Olivier Pirson
 * http://www.opimedia.be/
 *
 * @package OPiQuotations
 */
namespace OPiQuotations;

require_once 'OPiQuotations/log.inc';
require_once 'OPiQuotations/OPiQuotation.inc';


/**
 * Class to connect and deal to the MySQL database.
 */
class Db {
  /**
   * \brief
   * Open the connection to the database $db_name.
   *
   * If connection fails
   * then write a message in the errors log file.
   *
   * See is_connected() function.
   *
   * @param string $host Host name or IP address of the server
   * @param string $user MySQL user name
   * @param string $password MySQL password
   * @param string $db_name Name of the database
   */
  public function __construct($host, $user, $password, $db_name) {
    #DEBUG
    assert('is_string($host)');
    assert('is_string($user)');
    assert('is_string($password)');
    assert('is_string($db_name)');
    #DEBUG_END

    \mysqli_report(MYSQLI_REPORT_OFF);
    #DEBUG
    assert(\mysqli_report(MYSQLI_REPORT_ALL^MYSQLI_REPORT_STRICT^MYSQLI_REPORT_INDEX) === true);
    #DEBUG_END

    $this->connect = new \mysqli($host, $user, $password, $db_name);

    if ( $this->connect->connect_error ) {
      to_log('mysqli connect error '.\mysqli_connect_errno().' : '.\mysqli_connect_error().'
'.@print_r($this->connect, true));
      $this->connect = null;
    } elseif ( !$this->connect->set_charset('utf8') ) {
      to_log('mysqli::set_charset(\'utf8\') failed!
Server:'.$this->connect->server_info.'
Client:'.$this->connect->client_info.'
'.@print_r($this->connect, true));
      $this->connect->close();
      $this->connect = null;
    }
  }


  /**
   * \brief
   * Close the connection to the database.
   */
  public function __destruct() {
    if ( !empty($this->connect) ) {
      $this->connect->close();
    }
  }



  /**
   * \brief
   * If connection is ok
   * then return true
   * else return false.
   *
   * @return bool
   */
  public function is_connected() {
    return !empty($this->connect);
  }


  /**
   * \brief
   * Return the string with its special characters escaped
   * for use in a SQL statement.
   *
   * @param string $s
   *
   * @return string
   */
  public function escape($s) {
    #DEBUG
    assert('is_string($s)');
    assert('$this->is_connected()');
    #DEBUG_END

    return $this->connect->real_escape_string($s);
  }


  /**
   * \brief
   * Return a associative table id => array(name, number of use)
   * of elements of the table.
   *
   * @param string $table (must be 'author', 'nation', 'subject' or 'work')
   *
   * @return array[array]
   */
  public function list_to_assoc($table) {
    #DEBUG
    assert('is_string($table)');
    assert('in_array($table, array(\'author\', \'nation\', \'subject\' or \'work\'))');
    #DEBUG_END

    $quots = array();

    if (!$this->is_connected()) {
      to_log('Db.list_to_assoc(\''.print_r($table, true).'\') impossible because is NOT connected!');

      return $quots;
    }

    $id_names = array('author' => 'nation_author_id',
                      'nation' => 'nation_author_id');
    $id_name = (isset($id_names[$table])
                ? $id_names[$table]
                : $table.'_id');
    $is_maxim = ($table === 'nation'
                 ? ' AND `q`.`is_maxim`=1'
                 : ($table === 'author' || $table === 'work'
                    ? ' AND `q`.`is_maxim`=0'
                    : ''));

    $query = 'SELECT `t`.`id`, `t`.`name`, (SELECT COUNT(*) FROM `quotation` AS `q` WHERE `t`.`id`=`q`.`'.$id_name.'`'.$is_maxim.') AS `nb`
FROM `'.$table.'` AS `t`
ORDER BY `name` COLLATE utf8_unicode_ci;';

    $result = $this->connect->query($query);
    if ( $result !== false ) {
      while ($row = $result->fetch_assoc()) {
        settype($row['id'], 'int');
        settype($row['nb'], 'int');
        if ($row['nb'] > 0) {
          $quots[$row['id']] = array($row['name'], $row['nb']);
        }
      }

      $result->free_result();
    }
    else {
      to_log('MySQL Db.list_to_assoc
'.$query.'
failed!
Server:'.$this->connect->server_info.'
Client:'.$this->connect->client_info.'
'.@print_r($this->connect, true));
    }

    return $quots;
  }


  /**
   * \brief
   * Return the numbers of quotations and/or maxims.
   *
   * If $is_maxim === null then return the numbers of quotations/maxims,\n
   * if $is_maxim === false then return the numbers of quotations,\n
   * if $is_maxim === true then return the numbers of maxims.
   *
   * @param null|bool $is_maxim
   *
   * @return int >= 0
   */
  public function nb($is_maxim=null) {
    #DEBUG
    assert('($is_maxim === null) || is_bool($is_maxim)');
    assert('$this->is_connected()');
    #DEBUG_END

    if (!$this->is_connected()) {
      to_log('Db.nb(\''.print_r($is_maxim, true).'\') impossible because is NOT connected!');

      return 0;
    }

    $query = 'SELECT COUNT(*) AS `nb`
FROM `quotation` AS `q`';
    if ($is_maxim !== null) {
      $query .= '
WHERE `q`.`is_maxim`='.($is_maxim
                        ? '1'
                        : '0');
    }
    $query .= ';';

    $result = $this->connect->query($query);
    if ( $result !== false ) {
      $row = $result->fetch_assoc();

      return (int)$row['nb'];
    }
    else {
      to_log('MySQL query
'.$query.'
failed!
Server:'.$this->connect->server_info.'
Client:'.$this->connect->client_info.'
'.@print_r($this->connect, true));

      return 0;
    }
  }


  /**
   * \brief
   * Execute the MySQL query INSERT.
   *
   * If insertion is ok
   * then return true,
   * else return false.
   *
   * @param string $query Valid MySQL query
   *
   * @return bool
   */
  public function query_insert($query) {
    #DEBUG
    assert('is_string($query)');
    assert('$this->is_connected()');
    #DEBUG_END

    if (!$this->is_connected()) {
      to_log('Db.query_insert(\''.print_r($query, true).'\') impossible because is NOT connected!');

      return $quots;
    }

    $r = $this->connect->query($query);

    if (!$r) {
      to_log('MySQL Db.query_insert
'.$query.'
failed!
Server:'.$this->db->connect->server_info.'
Client:'.$this->db->connect->client_info.'
'.@print_r($this->db->connect, true));
    }

    return $r;
  }


  /**
   * \brief
   * Return an array with all OPiQuotation
   * that match with the SQL $where condition
   * in order specified by $order.
   *
   * @warning Use escape() with each string piece of $where if necessary.
   *
   * @param string $where Valid WHERE clause of the SELECT MySQL command used
   *        (see http://dev.mysql.com/doc/refman/5.1/en/select.html )
   * @param string $order Valid ORDER clause of the SELECT MySQL command used
   *        (see http://dev.mysql.com/doc/refman/5.1/en/select.html )
   *
   * @return OPiQuotation[]
   */
  public function query_quotations($where, $order='') {
    #DEBUG
    assert('is_string($where)');
    assert('is_string($order)');
    assert('$this->is_connected()');
    #DEBUG_END

    $quots = array();

    if (!$this->is_connected()) {
      to_log('Db.query_quotations(\''.print_r($where, true).'\', \''.print_r($order, true).'\') impossible because is NOT connected!');

      return $quots;
    }

    $query = array('SELECT `id`, `text`, `translation`, `is_maxim`, `is_marked`, `subject`, `nation`, `author`, `work`',
                   'FROM `vw_quotation`');
    $where = (string)$where;
    if ($where !== '') {
      $query[] = $where;
    }
    $order = (string)$order;
    if ($order !== '') {
      $query[] = $order;
    }

    $query = implode('
', $query).';';

    $result = $this->connect->query($query);
    if ( $result !== false ) {
      while ($row = $result->fetch_assoc()) {
        settype($row['id'], 'int');
        settype($row['is_maxim'], 'bool');
        settype($row['is_marked'], 'bool');

        $quots[] = new OPiQuotation($row['id'], $row['text'], $row['is_maxim'], $row['is_marked'],
                                    $row['translation'],
                                    $row['subject'], ($row['is_maxim']
                                                      ? $row['nation']
                                                      : $row['author']), $row['work']);
      }

      $result->free_result();
    }
    else {
      to_log('MySQL Db.query_quotations
'.$query.'
failed!
Server:'.$this->connect->server_info.'
Client:'.$this->connect->client_info.'
'.@print_r($this->connect, true));
    }

    return $quots;
  }



  /** @var \mysqli $connect
   * \brief
   * Connection to the MySQL database.
   */
  protected $connect;
}


return true;

?>