<?php /* -*- coding: utf-8 -*- */

/** \file OPiQuotation.inc
 * (October 25, 2015)
 *
 * \brief
 * Class quotation (text, author…) or maxim (text, nation…).
 *
 * Piece of OPiQuotations.
 * https://bitbucket.org/OPiMedia/opiquotations
 *
 * GPLv3 --- Copyright (C) 2014, 2015 Olivier Pirson
 * http://www.opimedia.be/
 *
 * @package OPiQuotations
 */
namespace OPiQuotations;


/**
 * \brief
 * Class quotation (text, author…) or maxim (text, nation…).
 */
class OPiQuotation {
  /**
   * \brief
   * Construct a quotation/maxim.
   *
   * See to_html().
   *
   * @param int $id Id of the quotation (must be > 0)
   * @param string $text Text of the quotation (should not contain \htmlonly'@@@#'\endhtmlonly and \htmlonly'#@@@'\endhtmlonly)
   * @param bool $is_maxim true if maxim (of a possibly nation), false if quotation (of an possibly author)
   * @param bool $is_marked true if marked quotation/maxim, else false
   * @param null|string $translation Possibly translation of the text (should not contain \htmlonly'@@@#'\endhtmlonly and \htmlonly'#@@@'\endhtmlonly)
   * @param null|string $subject Possibly subject
   * @param null|string $nation_author Possibly nation if maxim, else possibly author
   * @param null|string $work Possibly work if quotation (must be null if $is_maxim)
   */
  public function __construct($id, $text,
                              $is_maxim=false,
                              $is_marked=false,
                              $translation=null,
                              $subject=null, $nation_author=null, $work=null) {
    #DEBUG
    assert('is_int($id)');
    assert('$id > 0');
    assert('is_string($text)');
    assert('is_bool($is_maxim)');
    assert('is_bool($is_marked)');
    assert('($translation === null) || is_string($translation)');
    assert('($subject === null) || is_string($subject)');
    assert('($nation_author === null) || is_string($nation_author)');
    assert('($work === null) || is_string($work)');
    #DEBUG_END

    $this->id = $id;
    $this->text = $text;
    $this->maxim = $is_maxim;
    $this->marked = $is_marked;
    $this->translation = $translation;
    $this->subject = $subject;
    if ($is_maxim) {
      #DEBUG
      assert('$work === null');
      #DEBUG_END

      $this->nation = $nation_author;
    }
    else {
      $this->author = $nation_author;
      $this->work = $work;
    }
  }



  /**
   * \brief
   * Return $s formatted in HTML to be displayed.
   *
   * - HTML tags &lt;b>, &lt;c>, &lt;i>, &lt;sub>, &lt;sup> are used (the other tags are displayed as it is).
   * - The special character ¨ (diaeresis, U+00A8) is used to justify to the right.
   * - Beginning (at the beginning of the line and after an end of line) double space is used to indentation.
   *
   * @param string $s (should not contain \htmlonly'@@@#'\endhtmlonly and \htmlonly'#@@@'\endhtmlonly)
   *
   * @return string
   */
  private static function _format_correct_html($s) {
    #DEBUG
    assert('is_string($s)');
    # assert('preg_match(\'/ $/\', $s) === 0');  // check if no final space
    #DEBUG_END

    $s = preg_replace('/<(b|c|i|sub|sup|\/b|\/c|\/i|\/sub|\/sup)>/', '@@@#$1#@@@', $s);  // protect some HTML tags

    $s = htmlspecialchars($s);  // escape special HTLM characters

    // Restore HTML tags previously protected
    $s = preg_replace('/@@@#(b|i|sub|sup|\/b|\/i|\/sub|\/sup)#@@@/', '<$1>', $s);
    $s = preg_replace('/@@@#c#@@@/', '<span class="block-center">', $s);
    $s = preg_replace('/@@@#c#@@@/', '<span class="block-center">', $s);
    $s = preg_replace('/@@@#\/c#@@@\n?/', '</span>', $s);

    // Special character ¨ to justify this line to the right
    $s = preg_replace('/¨(.+)\n?/', '<span class="block-right">$1</span>', $s);

    // Non-breaking space with after « and before », ! and ?
    $s = preg_replace('/« /', '«&nbsp;', $s);
    $s = preg_replace('/ (»|!|\?)/', '&nbsp;$1', $s);

    // Non-breaking thin space before : and ;
    $s = preg_replace('/ (:|;)/', '<span class="nowrap">&thinsp;$1</span>', $s);

    // Spaces
    $s = preg_replace('/(^|\n)  /', '$1&emsp;&emsp;', $s);
    $s = preg_replace('/  /', '&nbsp;&nbsp;', $s);
    $s = preg_replace('/(^|\n) /', '&nbsp;', $s);

    // HTML tag to newline
    $s = preg_replace('/\n/', '<br>', $s);

    #DEBUG
    # assert('preg_match(\'/¨/\', $s) === 0');  // check if no character ¨
    # assert('preg_match(\'/\\\'/\', $s) === 0');  // check if no character ' (use ’ instead)
    #DEBUG_END

    return $s;
  }


  /**
   * \brief
   * If $search !== null
   * then each occurence of $search is surrounded by '&lt;span class="highlight">' and '&lt;/span>'.
   *
   * @param string $s
   * @param null|string $search
   *
   * @return string
   */
  private static function _highlight_html($s, $search=null) {
    #DEBUG
    assert('is_string($s)');
    assert('($search === null) || is_string($search)');
    #DEBUG_END

    if ($search !== null && $search !== '') {
      $search = preg_replace('/\n/', '<br>', $search);  // split on HTML tags

      $pieces = preg_split('/(<.+?()>|&.+?;)/', $s, -1, PREG_SPLIT_DELIM_CAPTURE);  // (added () in regexp to avoid bug in PHPstripDEBUG)
      foreach ($pieces as &$piece) {
        if ((preg_match('/^<.+>$/', $piece) !== 1) && (preg_match('/^&.+;$/', $piece)) !== 1) {  // it's NOT a HTML tag or entity
          $piece = preg_replace('/('.preg_quote($search, '/').')/i', '<span class="highlight">$1</span>', $piece);
        }
      }

      return implode($pieces);
    }
    else {
      return $s;
    }
  }



  /**
   * \brief
   * Return null or the author of the quotation.
   *
   * @return null|string
   */
  public function author() {
    return ($this->maxim
            ? null
            : $this->author);
  }


  /**
   * \brief
   * Return the id of the quotation/maxim.
   *
   * @return id > 0
   */
  public function id() {
    return $this->id;
  }


  /**
   * \brief
   * Return true if is a maxim (of a possibly nation),
   * false if is a quotation (of a possibly author).
   *
   * @return bool
   */
  public function is_maxim() {
    return $this->maxim;
  }


  /**
   * \brief
   * Return true if is a marked quotation/maxim,
   * else false.
   *
   * @return bool
   */
  public function is_marked() {
    return $this->marked;
  }


  /**
   * \brief
   * Return null or the nation of the maxim.
   *
   * @return null|string
   */
  public function nation() {
    return ($this->maxim
            ? $this->nation
            : null);
  }


  /**
   * \brief
   * Return null or the subject of the quotation/maxim.
   *
   * @return null|string
   */
  public function subject() {
    return $this->subject;
  }


  /**
   * \brief
   * Return the text of the quotation/maxim.
   *
   * @return string
   */
  public function text() {
    return $this->text;
  }


  /**
   * \brief
   * Return the complete quotation/maxim in a HTML format.
   *
   * Layout
   * - for a quotation:
   *\code
id
   subject  (mark)
text | translation
            author
              work
   *\endcode
   *
   * - for a maxim:
   *\code
id
   subject  (mark)
text | translation
            nation
   *\endcode
   *
   * In text and translation:
   * - HTML tags &lt;b>, &lt;c>, &lt;i>, &lt;sub>, &lt;sup> are used (the other tags are displayed as it is).
   * - The special character ¨ (diaeresis, U+00A8) is used to justify to the right.
   * - Beginning (at the beginning of the line and after an end of line) double space is used to indentation.
   *
   * If $search !== null
   * then each occurence of $search founded is surrounded by '&lt;span class="highlight">' and '&lt;/span>'.
   *
   * If $add_link
   * then add links to id, subject, nation, author and work.
   *
   * If $link_target
   * then add a target attribut to each link.
   *
   * The subject is surrounded by $subject_tag.
   *
   * $uneOPiCitation_link is use to the external id link.
   *
   * @param null|string $search
   * @param bool $add_link
   * @param null|string $link_target
   * @param string $subject_tag
   * @param string $uneOPiCitation_link
   *
   * @return string
   */
  public function to_html($search=null, $add_link=true, $link_target=null,
                          $subject_tag='h2', $uneOPiCitation_link='uneOPiCitation.php') {
    #DEBUG
    assert('($search === null) || is_string($search)');
    assert('is_bool($add_link)');
    assert('($link_target === null) || is_string($link_target)');
    assert('is_string($subject_tag)');
    assert('is_string($uneOPiCitation_link)');
    #DEBUG_END

    $link_target = ($link_target
                    ? ' target="'.$link_target.'"'
                    : '');

    // Maxim or quotation
    $is_marked = ($this->is_marked()
                  ? ' marked'
                  : '');

    $a = array($this->is_maxim()
               ? '<section id="maxim-'.$this->id().'" class="maxim'.$is_marked.'">'
               : '<section id="quotation-'.$this->id().'" class="quotation'.$is_marked.'">');

    unset($is_marked);

    $a[] = '  <header>';

    // Marked ?
    if ($this->is_marked()) {
      $a[] = '    <div class="mark"></div>';
    }

    // Id
    $a[] = '    <div class="id">'.($add_link
                                   ? ('<a href="./?id='.$this->id().'"'.$link_target.'>'.$this->id()
                                      .'</a><a href="'.$uneOPiCitation_link.'?id='.$this->id()
                                      .'" target="_blank">&#8599;&#10102;</a>')
                                   : $this->id()).'</div>';

    if ($this->subject() !== null) {  // subject
      $html = $this->_highlight_html(htmlspecialchars($this->subject()), $search);
      $a[] = '    <'.$subject_tag.' class="subject">'.($add_link
                                                       ? '<a href="./?subject='.rawurlencode($this->subject()).'"'.$link_target.'>'.$html.'</a>'
                                                       : $html).'</'.$subject_tag.'>';
    }

    $a[] = '  </header>';

    if ($this->translation() === null) {  // text
      $a[] = '  <div class="text"><cite>'.$this->_highlight_html($this->_format_correct_html($this->text()), $search).'</cite></div>';
    }
    else {                                // text, translation
      $a[] = '  <div class="text_translation">
    <div class="text"><cite>'.$this->_highlight_html($this->_format_correct_html($this->text()), $search).'</cite></div>
    <div class="translation"><cite>'.$this->_highlight_html($this->_format_correct_html($this->translation()), $search).'</cite></div>
    <div class="clear"></div>
  </div>';
    }

    $a[] = '  <footer>';

    if ($this->is_maxim()) {  // nation
      #DEBUG
      assert('$this->author() === null');
      assert('$this->work() === null');
      #DEBUG_END

      if ($this->nation() !== null) {
        $html = $this->_highlight_html(htmlspecialchars($this->nation()), $search);
        $a[] = '    <div class="nation">'.($add_link
                                           ? '<a href="./?nation='.rawurlencode($this->nation()).'"'.$link_target.'>nation '.$html.'</a>'
                                           : 'nation '.$html).'</div>';
      }
    }
    else {                    // author, work
      #DEBUG
      assert('$this->nation() === null');
      #DEBUG_END

      if ($this->author() !== null) {
        $html = $this->_highlight_html(htmlspecialchars($this->author()), $search);
        $a[] = '    <div class="author">'.($add_link
                                           ? '<a href="./?author='.rawurlencode($this->author()).'"'.$link_target.'>'.$html.'</a>'
                                         : $html).'</div>';
      }

      if ($this->work() !== null) {
        $html = $this->_highlight_html(htmlspecialchars($this->work()), $search);
        $a[] = '    <div class="work">'.($add_link
                                         ? '<a href="./?work='.rawurlencode($this->work()).'"'.$link_target.'>'.$html.'</a>'
                                         : $html).'</div>';
      }
    }

    $a[] = '  </footer>';
    $a[] = '</section>
';

    return implode('
', $a);
  }


  /**
   * \brief
   * Return the quotation/maxim with its author/nation and work
   * in text format
   * (strip HTML tags &lt;b>, &lt;c>, &lt;i>, &lt;sub>, &lt;sup>
   * and special character ¨ (diaeresis, U+00A8)).
   * @return string
   */
  public function to_text() {
    $quot = trim($this->text());

    $quot = preg_replace('/<(b|c|i|sub|sup|\/b|\/c|\/i|\/sub|\/sup)>/', '', $quot);  // strip some HTML tags
    $quot = preg_replace('/¨/', '', $quot);  // strip special character ¨

    $quot = '"'.$quot.'"';

    if ($this->is_maxim()) {
      if ($this->nation() !== null) {  // add nation
        $quot .= '
(nation '.$this->nation().')';
      }
    }
    else {
      if ($this->author() !== null) {
        if ($this->work() !== null) {  // add author and work
          $quot .= '
('.$this->author().'/
'.$this->work().')';
        }
        else {                         // add author
          $quot .= '
('.$this->author().')';
        }
      }
      else if ($this->work() !== null) {  // add work
        $quot .= '
('.$this->work().')';
      }
    }

    return $quot;
  }


  /**
   * \brief
   * Return the quotation/maxim with its author/nation and work
   * in text format to post to Facebook.
   *
   * If url !== ''
   * then add url to the end on a newline.
   *
   * @param string $url
   *
   * @return string
   */
  public function to_text_facebook($url='') {
    #DEBUG
    assert('is_string($url)');
    #DEBUG_END

    $quot = $this->to_text();

    if ($url !== '') {
      $url = '
'.$url;
    }

    return $quot.$url;
  }


  /**
   * \brief
   * Return the quotation/maxim with its author/nation and work
   * in short text format to post to Twitter.
   *
   * If a cutting is required then reduce contiguous whitespaces.
   *
   * Cut the result to not be longer than $max_length characters.
   * See
   * https://dev.twitter.com/docs/tco-link-wrapper/faq#How_do_I_calculate_if_a_Tweet_with_a_link_is_going_to_be_over_140_characters_or_not
   *
   * If url !== null
   * then add url to the end on a newline.
   *
   * If $short_url_length === null
   * then use the real length of the url,
   * else assume that the url will be replaced by a short url not be longer than $short_url_length.
   *
   * @param null|string $url
   * @param null|int $short_url_length
   * @param int $max_length (must be > 0)
   *
   * @return string
   */
  public function to_text_twitter($url=null, $short_url_length=null, $max_length=140) {
    #DEBUG
    assert('($url === null) || is_string($url)');
    assert('($short_url_length === null) || is_int($short_url_length)');
    assert('($short_url_length === null) || ($short_url_length >= 0)');
    assert('is_int($max_length)');
    assert('$max_length > 0');
    #DEBUG_END

    $quot = $this->to_text();

    if ($url !== null) {
      if ($short_url_length === null) {
        $short_url_length = mb_strlen($url);
      }
      $max_length -= 1 + $short_url_length;

      $url = '
'.$url;
    }

    return text_cut($quot, $max_length).$url;
  }


  /**
   * \brief
   * Return null or the translation of the quotation/maxim.
   *
   * @return null|string
   */
  public function translation() {
    return $this->translation;
  }


  /**
   * \brief
   * Return null or the work of the quotation.
   *
   * @return null|string
   */
  public function work() {
    return ($this->maxim
            ? null
            : $this->work);
  }



  /** @var null|string $author
   * \brief
   * null or the author of the quotation if is a quotation,
   * else null.
   */
  protected $author;

  /** @var int $id
   * \brief
   * The id of the quotation/maxim (must be > 0).
   */
  protected $id;

  /** @var bool $is_marked
   * \brief
   * true if marked quotation/maxim,
   * else false.
   */
  protected $is_marked;

  /** @var bool $is_maxim
   * \brief
   * true if maxim (of a possibly nation),
   * false if quotation (of an possibly author)
   */
  protected $is_maxim;

  /** @var null|string $nation
   * \brief
   * null or the nation of the maxim if is a maxim,
   * else null.
   */
  protected $nation;

  /** @var null|string $subject
   * \brief
   * null or the subject of the quotation/maxim.
   */
  protected $subject;

  /**
   * @var string $text
   * \brief
   * Text of the quotation/maxim.
   *
   * Maybe contains some HTML tags (see __construct()).
   */
  protected $text;

  /**
   * @var null|string $translation
   * \brief
   * null or the translation of the quotation/maxim.
   *
   * Maybe contains some HTML tags (see __construct()).
   */
  protected $translation;

  /** @var string null|$work
   * \brief
   * null or the work of the quotation if is a quotation,
   * else null.
   */
  protected $work;
}


return TRUE;

?>