<?php /* -*- coding: utf-8 -*- */

/** \file OPiQuotations.inc
 * (October 26, 2015)
 *
 * \brief
 * Main class.
 *
 * PHP package to deal quotations (text, author…) and maxims (text, nation…)
 * from a MySQL database.
 *
 * Piece of OPiQuotations.
 * https://bitbucket.org/OPiMedia/opiquotations
 *
 * GPL3 --- Copyright (C) 2014, 2015 Olivier Pirson
 * http://www.opimedia.be/
 *
 * @version 02.00.00 --- October 26, 2015
 * @author Olivier Pirson <olivier_pirson_opi@yahoo.fr>
 * @package OPiQuotations
 *
 * \mainpage OPiQuotations
 * PHP package to deal quotations (text, author…) and maxims (text, nation…)
 * from a MySQL database.
 *
 * <ul>
 *   <li>Complete sources (and MySQL quotations file) on Bitbucket: <a href="https://bitbucket.org/OPiMedia/opiquotations" target="_blank"><tt>https://bitbucket.org/OPiMedia/opiquotations</tt></a></li>
 *   <li><a href="http://www.opimedia.be/DS/webdev/PHP/OPiQuotations/docs/index.html">Online HTML documentation</a></li>
 * </ul>
 *
 * <img src="OPiQuotations-64x64-t.png" width="64" height="64" border="0" alt="[OPiQuotations]" />
 *
 * My personal use of this package:
 *
 * \htmlonly
 * <a href="http://www.opimedia.be/OPiCitations/" target="_blank"><img src="OPiCitation-banner--577x100.jpg" width="577" height="100" border="0" alt="[OPiCitations]" /></a>
 * <ul>
 *   <li><a href="http://www.opimedia.be/OPiCitations/" target="_blank">OPiCitations <img src="OPiQuotations-16x16-t.png" width="16" height="16" border="0" alt="" /></a>: Web application containing over 3000 French quotes</li>
 *   <li><a href="http://www.opimedia.be/OPiCitations/uneOPiCitation.php" target="_blank">une O<span class="small-caps">Pi</span>Citation <img src="oneOPiQuotation-16x16-t.png" width="16" height="16" border="0" alt="" /></a>: Little Web application to display one OPiCitation (may be used in an iframe to include in another Web page)</li>
 *   <li><a href="https://twitter.com/OPiCitationJour" target="_blank">@OPiCitationJour</a>: Twitter account with an OPiCitation by day</li>
 *   <li><a href="https://www.facebook.com/1OPiCitationParJour" target="_blank">1OPiCitationParJour</a>: Page Facebook with another OPiCitation by day</li>
 * </ul>
 * \endhtmlonly
 *
 * \htmlonly
 * <hr />
 * <div>
 *   One quotation randomly chosen from the dictionary of French quotes:
 *   <a href="http://www.opimedia.be/OPiCitations/uneOPiCitation.php" target="_blank"><strong>une O<span class="small-caps">Pi</span>Citations</strong> <img src="oneOPiQuotation-16x16-t.png" width="16" height="16" border="0" alt="" /></a>.
 *   <iframe src="http://www.opimedia.be/OPiCitations/uneOPiCitation.php?no-link-OPiQuotations" width="100%" height="200" scrolling="auto" frameborder="0" onload="this.height=this.contentWindow.document.getElementsByTagName('body')[0].clientHeight;">[Your browser does not support iframe.]</iframe>
 * </div>
 * <hr />
 * \endhtmlonly
 *
 * <div>
 * GPLv3
 * ------
 * Copyright (C) 2014, 2015 Olivier Pirson
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * </div>
 * <hr>
 *
 * \htmlonly
 * <div align="center">
 *   <a class="net" href="http://www.opimedia.be/donate/" target="_blank"><img src="Donate-92x26-t.png" width="92" height="26" border="0" alt="[Donate]" /></a>
 * </div>
 * \endhtmlonly
 *
 * <div align="right">
 *   &copy; Olivier <span style="font-variant:small-caps">Pirson</span>\n
 *   <a class="net" href="http://www.opimedia.be/" target="_blank"><tt>www.opimedia.be</tt></a>\n
 *   <a class="mail" href="mailto:olivier_pirson_opi@yahoo.fr?subject=[OPiQuotations]"><tt>olivier_pirson_opi@yahoo.fr</tt></a>
 * </div>
 */

namespace OPiQuotations;


/**
 * \brief
 * If length of $html_text <= $max_length characters
 * then return $html_text,
 * else return $html_text cutted added of $hellip to not be longer than $max_length characters.
 *
 * If a cutting is required and $reduce then reduce contiguous whitespaces.
 *
 * If the cutting cut a HTML entity
 * then remove it.
 *
 * @param string $html_text
 * @param int $max_length (must be >= length of $hellip)
 * @param string $hellip; (must not containt the & character)
 * @param bool $reduce
 *
 * @return string
 */
function html_text_cut($html_text, $max_length, $hellip='…', $reduce=true) {
  #DEBUG
  assert('is_string($html_text)');
  assert('is_int($max_length)');
  assert('is_string($hellip)');
  assert('$max_length >= mb_strlen($hellip)');
  assert('is_bool($reduce)');
  #DEBUG_END

  if (mb_strlen($html_text) > $max_length) {
    $html_text = text_cut($html_text, $max_length, $hellip, $reduce);

    $i = mb_strrpos($html_text, '&');
    if (($i !== false) && (mb_strrpos($html_text, ';', $i + 1) === false)) {
      // Remove partial HTML entity like &quot;
      $html_text = mb_substr($html_text, 0, $i).$hellip;
    }
  }

  return $html_text;
}


/**
 * \brief
 * If length of $text <= $max_length characters
 * then return $text,
 * else return $text cutted added of $hellip to be length of $max_length characters.
 *
 * If a cutting is required and $reduce then reduce contiguous whitespaces.
 *
 * @param string $text
 * @param int $max_length (must be >= length of $hellip)
 * @param string $hellip;
 * @param bool $reduce
 *
 * @return string
 */
function text_cut($text, $max_length, $hellip='…', $reduce=true) {
  #DEBUG
  assert('is_string($text)');
  assert('is_int($max_length)');
  assert('is_string($hellip)');
  assert('$max_length >= mb_strlen($hellip)');
  assert('is_bool($reduce)');
  #DEBUG_END

  if (mb_strlen($text) > $max_length) {
    if ($reduce) {
      // Reduce contiguous whitespaces with break line
      $text = preg_replace('/\s*\n\s*/', '
', $text);

      // Reduce other contiguous whitespaces (\v is ignored because confusion with \n)
      $text = preg_replace('/[ \t\r\f]+/', ' ', $text);
    }

    $text = mb_substr($text, 0, $max_length - mb_strlen($hellip)).$hellip;
  }

  return $text;
}



/**
 * \brief
 * Class to get OPiQuotation and informations from the databse.
 */
class OPiQuotations {
  /**
   * \brief
   * Connect to the database.
   * See the Db class.
   */
  public function __construct() {
    require_once 'OPiQuotations/Db.inc';
    require_once 'OPiQuotations/.private/db_login.inc';

    $this->db = new Db($db_host, $db_user, $db_password, $db_name);
  }


  /**
   * \brief
   * Return a associative table id => array(name, number of use)
   * of elements of the table author.
   *
   * @return array[array]
   */
  public function list_authors() {
    return $this->db->list_to_assoc('author');
  }


  /**
   * \brief
   * Return a associative table id => array(name, number of use)
   * of elements of the table nation.
   *
   * @return array[array]
   */
  public function list_nations() {
    return $this->db->list_to_assoc('nation');
  }


  /**
   * Return a associative table id => array(name, number of use)
   * of elements of the table subject.
   *
   * @return array[array]
   */
  public function list_subjects() {
    return $this->db->list_to_assoc('subject');
  }


  /**
   * \brief
   * Return a associative table id => array(name, number of use)
   * of elements of the table work.
   *
   * @return array[array]
   */
  public function list_works() {
    return $this->db->list_to_assoc('work');
  }


  /**
   * \brief
   * If $is_maxim === null then return the numbers of quotations/maxims,
   * if $is_maxim === false then return the numbers of quotations,
   * if $is_maxim === true then return the numbers of maxims.
   *
   * @param null|bool $is_maxim
   *
   * @return int >= 0
   */
  public function nb($is_maxim=null) {
    #DEBUG
    assert('($is_maxim === null) || is_bool($is_maxim)');
    #DEBUG_END

    return $this->db->nb($is_maxim);
  }


  /**
   * \brief
   * Return the unique quotation/maxim of id $id.
   *
   * If this id doesn't exist
   * then return null.
   *
   * @param int $id
   *
   * @return null|OPiQuotation
   */
  public function quotation_by_id($id) {
    #DEBUG
    assert('is_int($id)');
    #DEBUG_END

    if ($id <= 0) {
      return null;
    }

    $quots = $this->db->query_quotations('WHERE `id`='.(int)$id.'');

    return (empty($quots)
            ? null
            : $quots[0]);
  }


  /**
   * \brief
   * Return the complete list of quotations/maxims.
   *
   * @return OPiQuotation[]
   */
  public function quotations_all() {
    return $this->db->query_quotations('',
                                       'ORDER BY `text_stripped` COLLATE utf8_unicode_ci');
  }


  /**
   * \brief
   * Return the list of quotations/maxims that are marked.
   *
   * @return OPiQuotation[]
   */
  public function quotations_all_marked() {
    return $this->db->query_quotations('WHERE `is_marked`',
                                       'ORDER BY `text_stripped` COLLATE utf8_unicode_ci');
  }


  /**
   * \brief
   * Return the list of quotations that are writed by author $author.
   *
   * @param string $author
   *
   * @return OPiQuotation[]
   */
  public function quotations_by_author($author) {
    #DEBUG
    assert('is_string($author)');
    #DEBUG_END

    return $this->db->query_quotations('WHERE NOT `is_maxim` AND `author` LIKE \'%'.$this->db->escape($author).'%\' COLLATE utf8_unicode_ci',
                                       'ORDER BY `author`, `text_stripped` COLLATE utf8_unicode_ci');
  }


  /**
   * \brief
   * Return the list of quotations/maxims of id among $ids.
   *
   * @param int[] $ids (each int > 0)
   *
   * @return OPiQuotation[]
   */
  public function quotations_by_ids($ids) {
    #DEBUG
    assert('is_array($ids)');
    #DEBUG_END

    foreach($ids as &$id) {
      #DEBUG
      assert('is_int($id)');
      assert('$id > 0');
      #DEBUG_END

      settype($id, 'int');
    }

    return (empty($ids)
            ? array()
            : $this->db->query_quotations('WHERE `id` IN ('.implode(',', $ids).')'));
  }


  /**
   * \brief
   * Return the list of maxims of nationality $nation.
   *
   * @param string $nation
   *
   * @return OPiQuotation[]
   */
  public function quotations_by_nation($nation) {
    #DEBUG
    assert('is_string($nation)');
    #DEBUG_END

    return $this->db->query_quotations('WHERE `is_maxim` AND `nation` LIKE \'%'.$this->db->escape($nation).'%\' COLLATE utf8_unicode_ci',
                                       'ORDER BY `nation`, `text_stripped` COLLATE utf8_unicode_ci');
  }


  /**
   * \brief
   * Return a list of $nb different quotations choose at random.
   *
   * If $label !== null
   * then choose quotations not already chosen with this label
   *      and set the chosen quotations with this $label.
   *
   * @param int $nb (must be >= 0)
   * @param null|string $label
   *
   * @return OPiQuotation[]
   */
  public function quotations_by_random($nb=1, $label=null) {
    #DEBUG
    assert('is_int($nb)');
    assert('$nb >= 0');
    assert('($label === null) || is_string($label)');
    #DEBUG_END

    if ($label === null) {
      return $this->db->query_quotations('',
                                         'ORDER BY RAND() LIMIT '.(int)$nb);
    }
    else {
      $label = $this->db->escape($label);
      $quots = $this->db->query_quotations('WHERE `id` NOT IN (SELECT `quotation_id` FROM `_selected_label` WHERE `label` = \''.$label.'\')',
                                           'ORDER BY RAND() LIMIT '.(int)$nb);

      if (!empty($quots)) {
        $query = array();
        foreach ($quots as $quot) {
          $query[] = '('.$quot->id().', \''.$label.'\', UTC_TIMESTAMP())';
        }
        $query = 'INSERT INTO `_selected_label` (`quotation_id`, `label`, `datetime_utc`) VALUES
'.implode(',
', $query).';';

        $this->db->query_insert($query);
      }

      return $quots;
    }
  }


  /**
   * \brief
   * Return the list of quotations/maxims of subject $subject
   *
   * @param string $subject
   *
   * @return OPiQuotation[]
   */
  public function quotations_by_subject($subject) {
    #DEBUG
    assert('is_string($subject)');
    #DEBUG_END

    return $this->db->query_quotations('WHERE `subject` LIKE \'%'.$this->db->escape($subject).'%\' COLLATE utf8_unicode_ci',
                                       'ORDER BY `subject`, `text_stripped` COLLATE utf8_unicode_ci');
  }


  /**
   * \brief
   * Return the list of quotations/maxims that contains the text $text.
   *
   * @param string $text
   *
   * @return OPiQuotation[]
   */
  public function quotations_by_text($text) {
    #DEBUG
    assert('is_string($text)');
    #DEBUG_END

    return $this->db->query_quotations('WHERE `text` LIKE \'%'.$this->db->escape($text).'%\' COLLATE utf8_unicode_ci',
                                       'ORDER BY `text_stripped` COLLATE utf8_unicode_ci');
  }


  /**
   * \brief
   * Return the list of quotations/maxims with a translation that contains the text $translation.
   *
   * @param string $translation
   *
   * @return OPiQuotation[]
   */
  public function quotations_by_translation($translation) {
    #DEBUG
    assert('is_string($translation)');
    #DEBUG_END

    return $this->db->query_quotations('WHERE `translation` LIKE \'%'.$this->db->escape($translation).'%\' COLLATE utf8_unicode_ci',
                                       'ORDER BY `translation_stripped` COLLATE utf8_unicode_ci');
  }


  /**
   * \brief
   * Return the list of quotations/maxims that are writed in work $work.
   *
   * @param string $work
   *
   * @return OPiQuotation[]
   */
  public function quotations_by_work($work) {
    #DEBUG
    assert('is_string($work)');
    #DEBUG_END

    return $this->db->query_quotations('WHERE NOT `is_maxim` AND `work` LIKE \'%'.$this->db->escape($work).'%\' COLLATE utf8_unicode_ci',
                                       'ORDER BY `work`, `text_stripped` COLLATE utf8_unicode_ci');
  }



  /** @var Db $db
   * \brief
   * Link to the database
   */
  protected $db;
}


return TRUE;

?>